/* ============================================================== */
/* This basic program illustrates the use of threads under UNIX.  */
/* This program is really basic, it just lauch N threads (that do */
/* nothing) and get their return values.                          */
/* ============================================================== */


#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <pthread.h>

/* ============================================================== */
/* The definition of the macro PTHREAD_THREADS_MAX is hard to     */
/* find! Under Linux, I have seen it in "bits/local_lim.h". Under */
/* SUN, I have seen in "limits.h", but the macro __EXTENSIONS__   */
/* must be defined. You may need to ajust these for your system.  */
/* ============================================================== */

#undef OS_FOUND

#ifdef LINUX
#include <bits/local_lim.h>
#define OS_FOUND
#endif

#ifdef SUN
#define __EXTENSIONS__
include <limits.h>
#define OS_FOUND
#endif

#ifndef OS_FOUND
#error "You should define LINUX or SUN!!!"
#endif

/* ============================================================== */
/* Threads' arguments                                             */
/* ============================================================== */

static int *thread_arguments;

/* ============================================================== */
/* Threads' returns values                                        */
/* ============================================================== */

static int *thread_return_values;

/* ============================================================== */
/* Threads' function                                              */
/* Please note that the 'return' statement will not be executed   */
/* by the threads' function because pthread_exit() terminates the */
/* thread's execution.                                            */
/* ============================================================== */

void* Thread_Proc(void* thread_parameter)
{
  pthread_exit( (void*)&(thread_return_values[*((int*)thread_parameter)]) );
  return (void*)NULL;
}







int main (int argc, char *argv[])
{
   /* =========================================================== */
   /* number_of_threads:                                          */
   /* Number of threads to launch.                                */
   /*                                                             */
   /* thread_ids:                                                 */
   /* Array that contains all the threads' identifiers.           */ 
   /*                                                             */
   /* thread_attributs:                                           */
   /* Array that contains all the threads' attributs.             */
   /*                                                             */
   /* thread_return_value:                                        */
   /* Pointer used to store the thread's return value.            */
   /* =========================================================== */

   int             number_of_threads;
   pthread_t       *thread_ids;
   pthread_attr_t  thread_attributs;
   void            *thread_return_value;

   int             i, cr;



   /* =========================================================== */
   /* Check the syntax of the command line                        */
   /* =========================================================== */

   if (argc != 2)
   {
     fprintf (stderr, "\nUsage: threads.test <number of threads>\n");
     return 1;
   }

   number_of_threads = atoi(argv[1]);

   /* =========================================================== */
   /* The maximum number of thread per process is defined by the  */
   /* macro PTHREAD_THREADS_MAX.                                  */
   /* =========================================================== */

   if (number_of_threads > PTHREAD_THREADS_MAX)
   {
     fprintf (stderr, "\nthreads.test error: maximum number of threads is %d\n", PTHREAD_THREADS_MAX);
     return 1;
   }

   /* =========================================================== */
   /* Allocate array of threads' indentifiers                     */
   /* =========================================================== */

   thread_ids = (pthread_t*)malloc( number_of_threads * sizeof(pthread_t) );
   if (thread_ids == NULL)
   {
     fprintf (stderr, "\nthreads.test error: can not allocate memory!\n");
     return 1;
   }

   /* =========================================================== */
   /* Allocate array for threads' return values                   */
   /* =========================================================== */

   thread_return_values = (int*)malloc ( number_of_threads * sizeof(int) );
   if (thread_return_values == NULL)
   {
     fprintf (stderr, "\nthreads.test error: can not allocate memory!\n");
     return 1;
   }

   /* =========================================================== */
   /* Allocate array for threads' arguments                       */
   /* =========================================================== */

   thread_arguments = (int*)malloc ( number_of_threads * sizeof(int) );
   if (thread_return_values == NULL)
   {
     fprintf (stderr, "\nthreads.test error: can not allocate memory!\n");
     return 1;
   }

   /* =========================================================== */
   /* Initialise global variables                                 */
   /* =========================================================== */

   for (i=0; i<number_of_threads; i++)
   {
     thread_arguments[i]     = i;
     thread_return_values[i] = i*10;
   }

   /* =========================================================== */
   /* Set the thread's attributs                                  */
   /* (1) Set the thread's attributs to default values.           */
   /* (2) Set the thread to "joinable state". Another thread      */
   /*     can synchronize on the thread termination and reco-     */
   /*     -ver its termination code using pthread_join(). But     */
   /*     some of the thread resources are kept allocated af-     */
   /*     -ter the thread terminates. Only the call to the        */
   /*     function pthread_join() can free these ressources.      */
   /*                                                             */
   /* Note: Attribute objects are consulted only when creating a  */
   /*       new thread. The same attribute object can be used     */
   /*       for creating several threads.                         */
   /* =========================================================== */

   if (pthread_attr_init(&thread_attributs) != 0)
   {
     fprintf (stderr, "\nthreads.test error: pthread_attr_init() failed\n");
     return 1;
   }

   if (pthread_attr_setdetachstate(&thread_attributs, PTHREAD_CREATE_JOINABLE) != 0)
   {
     fprintf (stderr, "\nthreads.test error: pthread_attr_setdetachstate() failed\n");
     return 1;
   }

   /* =========================================================== */
   /* Launch the threads                                          */
   /* =========================================================== */

   for (i=0; i<number_of_threads; i++)
   {

     fprintf (stdout, "\nLaunching thread number %d", i+1);
     fflush (stdout);

     cr = pthread_create (
                           &(thread_ids[i]),
                           &thread_attributs,
                           Thread_Proc,
                           (void*)(&(thread_arguments[i]))
                         );

     if (cr != 0)
     {
       fprintf (stderr, "\nthreads.test error: pthread_create() failed\n");
       return 1;
     }
   }

   /* =========================================================== */
   /* Now get the return value for all threads                    */
   /* Please note that this free all the allocated memory used to */
   /* keep threads' status.                                       */
   /* =========================================================== */

   for (i=0; i<number_of_threads; i++)
   {
      if (pthread_join (thread_ids[i], &thread_return_value) != 0)
      {
        fprintf (stderr, "\nthreads.test error: pthread_join() failed\n");
        return 1;
      }

      fprintf (stdout, "\nReturn value for thread number %d is %d", i, *((int*)thread_return_value));
   }

   /* =========================================================== */
   /* Clean exit                                                  */
   /* =========================================================== */

   free (thread_ids);
   free (thread_return_values);
   free (thread_arguments);

   return 0;
}



