/*! \file chtbl.h
    Header file for chtbl.c
 */


#ifdef __cplusplus
extern "C" {
#endif


#ifndef CHTBL_H
#define CHTBL_H

#include <stdlib.h>
#include "list.h"


/* ---------------------------------------------------------------- */
/* Structure used to walk through a hash                            */
/* ---------------------------------------------------------------- */

/*! \brief This value is used by the function chtbl_next(). It tells the function to set the hash pointer to the
           first element of the hash.
 */

#define REWIND_HASH    0

/*! \brief This value is used by the function chtbl_next(). It tells the function to set the hash pointer to the
           current element of the hash.
 */

#define CURRENT_HASH   1

/*! \brief Data structure that defines one element within a hash table. This structure is used by the function
           chtbl_next().
 */

typedef struct CHTbl_handle_
{
  /*! \brief Index of the current linked list. */
  int        current_list;

  /*! \brief Pointer to the current element within the current linked list. */
  ListElmt   *current_element;

} CHTbl_handle;

/* ---------------------------------------------------------------- */
/* Data structures for the hash tables                              */
/* ---------------------------------------------------------------- */

/*! \brief Data structure that defines a linked hash table.
 */

typedef struct CHTbl_
{
  /*! \brief Number of linked list hold by the hash table. */
  int  conteneurs;

  /*! \brief Pointer to the function used to hash the input key. */
  int  (*h)(const void *cle);

  /*! \brief Pointer to the function used to compare the data pointed by two elements of the hash table. */
  int  (*corresp)(const void *cle1, const void *cle2);

  /*! \brief Pointer to the function used to free the memory allocated for the data pointed by one element of the hash table. */
  void (*detruire)(void *donnee);

  /*! \brief Number of element in the hash table. */
  int  taille;

  /*! \brief Pointer to a linked list. */
  List *table;

  /*! \brief This parameter represents the position of one element within the linked list. It is used bu the
             function chtbl_next(). */
  CHTbl_handle pointer;

} CHTbl;

/* ---------------------------------------------------------------- */
/* Hash table's API                                                 */
/* ---------------------------------------------------------------- */


int chtbl_init(CHTbl *htab, int conteneurs, 
              int (*h)(const void *cle), 
              int (*corresp)(const void *cle1, const void *cle2), 
              void (*detruire)(void *donnee));

void chtbl_destroy(CHTbl *htab);

int chtbl_insert(CHTbl *htab, const void *donnee);

int chtbl_remove(CHTbl *htab, void **donnee);

int chtbl_lookup(const CHTbl *htab, void **donnee);

int chtbl_find (const CHTbl *htab, void **donnee);

void* chtbl_next (CHTbl *htab, int rewind);

#define chtbl_size(htab) ((htab)->taille)

#endif

#ifdef __cplusplus
}
#endif

