/*! \file config.h
    Header file for config.c.
 */

#ifdef __cplusplus
extern "C" {
#endif

#ifndef CONFIG_HD

  #ifdef PRIVATE_CODE_ONLY

     /*! \brief Thei FLEX configuration file is not valid.
      */

     #define CONF_PARSER_SYNTAX_ERROR   -1

     /*! \brief The FLEX configuration parser could not allocate memory.
      */

     #define CONF_PARSER_NO_MEM         -2

     /*! \brief The FLEX configuration parser found a duplicated tag in the configuration file.
      */

     #define CONF_PARSER_DUPLICATED_TAG -3

     /*! \brief The configuration file has been successfully parsed.
      */

     #define CONF_PARSER_OK             0

     /*! \brief Data structure used to store the couples (tag, value).
      */

     struct user_data {
                         char *tag;
                         char *value;
                      };

  #endif

  /*! \brief configuration parser could not open the configuration file.
   */

  #define PARSE_CONFIGURATION_FILE_NOT_FOUND    -1

  /*! \brief configuration parser could not allocate memory.
   */

  #define PARSE_CONFIGURATION_NO_MEM            -2

  /*! \brief configuration parser found an invalid line.
   */

  #define PARSE_CONFIGURATION_SYNTAX_ERROR      -3

  /*! \brief configuration parser found a duplicated tag.
   */

  #define PARSE_CONFIGURATION_DUPLICATED_TAG    -4

  /*! \brief configuration parser terminated successfully.
   */

  #define PARSE_CONFIGURATION_OK                0


  int parse_configuration_file (char *path, int *err);
  void free_all_configuration_data();
  int get_value (char *tag, char **value);


  #define CONFIG_HD

#endif

#ifdef __cplusplus
}
#endif

