/*! \file cyclic_buffer.h
    Header file for cyclic_buffer.c
 */

#ifdef __cplusplus
extern "C" {
#endif


#ifndef CYCLIC_BUFFER_HD


#include <stdlib.h>
#include <string.h>


  /* ========================================================== */
  /* Data structure                                             */
  /* ========================================================== */

  /*! \brief Basic structure that defines the cyclic buffer. 
   */
  struct s_cyclic_buffer {
                           /*! \brief Array of pointers to (void*), each element points to an item of the buffer. */
                           void           **buffer;

                           /*! \brief Maximum number of elements in the buffer. */
                           unsigned long  size;

                           /*! \brief Index to the first item in the cyclic buffer. */
                           unsigned long  start;

                           /*! \brief Index to the last item in the cyclic buffer. */
                           unsigned long  stop;

                           /*! \brief Pointer to the current index. */
                           unsigned long  current;

                           /*! \brief Current number of element in the buffer (from 0 to 'size'). */ 
                           unsigned long  current_size;

                           /*! \brief Current number of element extracted (this is used by get_element()). */
                           unsigned long  n;

                         };

  typedef struct s_cyclic_buffer* cyclic_buffer;


  /* ========================================================== */
  /* Buffer functions                                           */
  /* ========================================================== */

  cyclic_buffer create_cyclic_buffer (unsigned long size);
  void destroy_cyclic_buffer (cyclic_buffer buffer);
  void* cyclic_buffer_add_element (cyclic_buffer buffer, void* element, unsigned element_size);
  void rewind_cyclic_buffer (cyclic_buffer buffer);
  void* cyclic_buffer_get_element(cyclic_buffer buffer);
  unsigned long cyclic_buffer_get_start(cyclic_buffer buffer);
  unsigned long cyclic_buffer_get_stop(cyclic_buffer buffer);
  unsigned long cyclic_buffer_get_current_size(cyclic_buffer buffer);

  #define CYCLIC_BUFFER_HD
#endif


#ifdef __cplusplus
}
#endif

