/*! \file files.h
    Header file for file.c.
 */

#ifdef __cplusplus
extern "C" {
#endif


#ifndef FILES_HD

#include <unistd.h>
#include <errno.h>
#include <stdlib.h>
#include <stdio.h>
#include <time.h>


#include "flock.h"

int wait_for_file (char* file_name, int period, char* stop_file);
char* wait_for_multiple_file (char **file_names, int period, char* stop_file);
int read_file (char *file_name, char **buff);
int set_timeout_file (char *filename);
int test_timeout_file (char *filename, int timeout);
void basename (char *src, char *dst);

/*! \brief This value is used by various functions that need to allocate buffers. This represents the initial size of the
           buffer (in bytes).
 */
#define FILES_UTIL_INIT_BUFF_SIZE   2048

/*! \brief Return value for the function read_file(). The function can not allocate memory.
 */
#define FILES_UTIL_MALLOC_ERR       -1

/*! \brief Return value for the function read_file(). An error occured while reading the input file.
 */
#define FILES_UTIL_READ_ERR         -2

/*! \brief Return value for the function read_file(). Can not open the input file.
 */
#define FILES_UTIL_OPEN_ERR         -3

/*! \brief Return value for the function set_timeout_file(). Can not open target file.
 */
#define SET_TIMEOUT_FILE_OPEN_ERROR   -4

/*! \brief Return value for the function set_timeout_file(). An error occured while writing into the target file.
 */
#define SET_TIMEOUT_FILE_WRITE_ERROR  -5 

/*! \brief Return value for the function set_timeout_file(). Timeout file set correctly.
 */
#define SET_TIMEOUT_FILE_OK           -6

/*! \brief Return value for the function test_timeout_file(). The timeout is passed.
 */ 
#define TEST_TIMEOUT_FILE_TIMEOUT_ELLAPSED  -7

/*! \brief Return value for the function test_timeout_file(). An error occured while reading the timeout file.
 */
#define TEST_TIMEOUT_FILE_READ_ERROR  -8

/*! \brief Return value for the function test_timeout_file(). The timeout is not passed (you may have to wait).
 */
#define TEST_TIMEOUT_FILE_TIMEOUT_NOT_ELLAPSED  -9

/*! \brief Return value for the function test_timeout_file(). The function could not delete the timeout file.
 */
#define TEST_TIMEOUT_FILE_UNLINK_FAILED -10

#define FILES_HD
#endif 

#ifdef __cplusplus
}
#endif

