/*! \file flock.h
    Header file for flock.c.
 */

#ifdef __cplusplus
extern "C" {
#endif

#ifndef FLOCK_HD

  #include <stdio.h>
  #include <unistd.h>
  #include <sys/types.h>
  #include <sys/stat.h>
  #include <fcntl.h>
  #include <string.h>
  #include <stdarg.h>

  #ifdef LINUX
  #include <sys/file.h>
  #endif

  #ifdef SUN

/*************************************************************************/
/* Note 1:                                                               */
/*                                                                       */
/* In the SunOS header file "sys/varargs.h" you can read that:           */
/*                                                                       */
/* Many compilation systems depend upon the use of special functions     */
/* built into the the compilation system to handle variable argument     */
/* lists and stack allocations.  The method to obtain this in SunOS      */
/* is to define the feature test macro "__BUILTIN_VA_ARG_INCR" which     */
/* enables the following special built-in functions:                     */
/*      __builtin_alloca                                                 */
/*      __builtin_va_alist                                               */
/*      __builtin_va_arg_incr                                            */
/* It is intended that the compilation system define this feature test   */
/* macro, not the user of the system.                                    */
/*                                                                       */
/* ...                                                                   */
/* Do not follow these rules ... I've tried and it does not work.        */
/*                                                                       */
/* Note 2:                                                               */
/*                                                                       */
/* In the SunOS' man page for "va_start()", they tell you to include the */
/* header file "sys/varargs.h". Don't do that because it is wrong. You   */
/* must include "stdarg.h" instead.                                      */
/*************************************************************************/

  #include <fcntl.h>

  #endif


  /*! \brief Size, in bytes, of the buffer used by copy_file_with_lock() for I/O operations.
   */

  #define COPY_IOBUFF_SIZE   4096

  /*! \brief Return value for the function my_logger(). This means that the argument 'buffer' is too small.
   */

  #define LOG_BUFFER_OVERFLOW  -1

  /*! \brief Return value for the function my_logger(). The function can not write to the file (which name is specified
             as first argument).
   */

  #define LOG_WRITE_ERROR      -2

  /*! \brief Return value for the function my_logger(). The function can not open the file (which name is specified
             as first argument).
   */

  #define LOG_OPEN_ERROR       -3

  /*! \brief Return value for the function my_logger(). The function can not close the file (which name is specified
             as first argument).
   */

  #define LOG_CLOSE_ERROR      -4

  /*! \brief Return value for the function my_logger(). The function can not lock the file (which name is specified
             as first argument).
   */

  #define LOG_LOCK_ERROR       -5

  /*! \brief Return value for the function my_logger(). The function succed. Everything is fine.
   */

  #define LOG_OK               -6

  /*! \brief Return value for the function copy_file_with_lock(). Can not open the source file.
   */

  #define COPY_SRC_OPEN_ERROR  -1

  /*! \brief Return value for the function copy_file_with_lock(). Can not open the destination file.
   */

  #define COPY_DST_OPEN_ERROR  -2

  /*! \brief Return value for the function copy_file_with_lock(). Lock on the destination file failed.
   */

  #define COPY_LOCK_ERROR      -3

  /*! \brief Return value for the function copy_file_with_lock(). Can not stat() the srouce file.
   */

  #define COPY_STAT_ERROR      -4

  /*! \brief Return value for the function copy_file_with_lock(). Error while reading the source file.
   */

  #define COPY_READ_ERROR      -5

  /*! \brief Return value for the function copy_file_with_lock(). Error while writing to the destination file.
   */

  #define COPY_WRITE_ERROR     -6

  /*! \brief Return value for the function copy_file_with_lock(). Error while unlocking the destinatination file.
   */

  #define COPY_UNLOCK_ERROR    -7

  /*! \brief Return value for the function copy_file_with_lock(). Error while closing the source file.
   */

  #define COPY_SRC_CLOSE_ERROR -8

  /*! \brief Return value for the function copy_file_with_lock(). Error while closing the destination file.
   */

  #define COPY_DST_CLOSE_ERROR -9

  /*! \brief Return value for the function copy_file_with_lock(). File successfully copied.
   */

  #define COPY_OK               0


  int lock_file(int fd);
  int unlock_file(int fd);
  int my_logger(char *file_name, char *buffer, int buffsize, char *format, ...);
  int append_file_with_lock (char *src, char *dst);

#define FLOCK_HD
#endif


#ifdef __cplusplus
}
#endif

