/*! \file my_dir.h
    Header file for dir.c.
 */

#ifdef __cplusplus
extern "C" {
#endif


#ifndef DIR_HD

#include <sys/types.h>
#include <dirent.h>
#include <stdlib.h>
#include <sys/stat.h>

#ifdef LINUX
#include <string.h>
#endif

#ifdef SUN
#include <strings.h>
#endif

  /*! \brief Maximum number of characters for a file name.
   */

  #define FILE_NUMBER 10

  /*! \brief Data structure used to store directory's entries.
   */

  struct file_list {
                     /*! \brief Array of zero terminated strings of characters. It contains all the directory's entries. */
                     char **list;

                     /*! \brief Total number of allocated strings in the array 'list'. */
                     int  total_alloc;

                     /*! \brief Number of entries in the directory (less ot equal to 'total_alloc') */
                     int  num;
                   }; 

  /*! \brief Return code for base_dir_name(): the function did not find the base name.
   */
  #define BASE_DIR_NOT_FOUND  0

  /*! \brief Return code for base_dir_name(): can not allocate memory.
   */
  #define BASE_DIR_NO_MEM     1

  /*! \brief Return code for base_dir_name(): the function found the base name.
   */
  #define BASE_DIR_FOUND      2


  int read_directory (char *dir_name, struct file_list *flist);
  int read_directory_only (char *dir_name, struct file_list *flist_out);
  void free_file_list(struct file_list *flist);
  int base_dir_name (char *dirname, char **res);

  #define DIR_HD
#endif

#ifdef __cplusplus
}
#endif


