/*! \file my_sem.h
    Header file for sem.c.
 */


#ifdef __cplusplus
extern "C" {
#endif

#ifndef SEMAPHORES_HD

  #include <sys/types.h>
  #include <sys/ipc.h>
  #include <sys/sem.h>

  /* =========================================================================== */
  /* WARNING                                                                     */
  /*                                                                             */
  /* Under new version of Linux, the union 'semun' is NOT defined by default.    */
  /* See the file '/usr/include/bits/sem.h' for details: if the macro            */
  /* _SEM_SEMUN_UNDEFINED is defined, that you must define 'semun' yourself.     */
  /*                                                                             */
  /* Under Solaris, you must define the union 'semun'. Please look at the man    */
  /* page for semctl().                                                          */
  /* =========================================================================== */

  #ifdef LINUX
    #ifdef _SEM_SEMUN_UNDEFINED
      union semun
      {
        int val;                          
        struct semid_ds *buf;             
        unsigned short int *array;     
        struct seminfo *__buf;    
      };
    #endif
  #endif

  #ifdef SUN
    union semun
    {
      int             val;
      struct semid_ds *buf;
      ushort_t        *array;
    };
  #endif 


  /*! \brief This macro represents the default semaphore's premission (applied at creation time). This means that we grant
             all access rights (Read/Write/Execute) to the process' user. In binary, 0x1C0, is 111 000 000.
   */
  #define SEM_PERMISSION 0x000001C0

  /* ========================================================= */
  /* Return value for create_semaphore_set()                   */
  /* ========================================================= */

  /*! \brief Return value for the function create_semaphore_set(). The function was successful. */
  #define CREATE_SEMAPHORE_OK              0

  /*! \brief Return value for the function create_semaphore_set(). An error occured while creating the set of semaphores. */
  #define CREATE_SEMAPHORE_CREATE_ERROR   -1

  /*! \brief Return value for the function create_semaphore_set().  An error occured while setting the semaphores' values. */
  #define CREATE_SEMAPHORE_SET_VAL_ERROR  -2

  /* ========================================================= */
  /* return value for get_semaphore_set()                      */
  /* ========================================================= */

  /*! \brief Return value for the function get_semaphore_set(). The function was successful. */
  #define GET_SEM_OK                      0

  /*! \brief Return value for the function get_semaphore_set(). The process already got the set of semaphores. */
  #define GET_SEM_ALREADY_GET             -1

  /*! \brief Return value for the function get_semaphore_set(). An error occured while getting the set of semaphores. */
  #define GET_SEM_ERROR                   -2                   

  /* ========================================================= */
  /* Return value for set_semaphore_values()                   */
  /* ========================================================= */

  /*! \brief Return value for the function set_semaphore_values(). The function was successful. */
  #define SET_SEM_VALUES_OK                0

  /*! \brief Return value for the function set_semaphore_values(). The semaphore set was not created (or get). */
  #define SET_SEM_VALUES_SEM_NOT_CREATED  -1

  /*! \brief Return value for the function set_semaphore_values(). An error occured while setting the semaphores' values. */
  #define SET_SEM_VALUES_SET_ERROR        -2

  /* ========================================================= */
  /* API's description                                         */
  /* ========================================================= */

  void change_sem_access_permission(int new_permission);
  int create_semaphore_set (key_t ipc_key, int number_of_sem);
  int set_semaphore_values (int *values);
  int set_semaphore_value (int value, int sem_number);
  int take_sem (int sem_idx);
  int release_sem (int sem_idx);
  int destroy_semaphore_set();
  int get_semaphore_set();

#endif

#ifdef __cplusplus
}
#endif

