/*! \file my_shm.h
    Header file for shm.c.
 */

#ifdef __cplusplus
extern "C" {
#endif

#ifndef SHM_HD

  #include <sys/types.h>
  #include <sys/ipc.h>
  #include <sys/shm.h>


  #ifdef SUN
  #define SHM_addr    void*
  #endif

  #ifdef LINUX
  #define SHM_addr    char*
  #endif

  #ifndef SHM_addr
  #error "ERROR: You must specify a build target (LINUX or SUN)."
  #endif

  /*! \brief Define default permissions for all newly created SHM. It should ne read just like file permissions:
             0x1C0 = 111000000 in binary, that is 111 000 000. This gives all rights for the user and nothing for other 
             users.
   */

  #define SHM_PERMISSION 0x000001C0

  /* ============================================================= */
  /* Return value for the function create_shared_memory_segment()  */
  /* ============================================================= */

  /*! \brief Return value for the function create_shared_memory_segment(). This means that the SHM has been successfully 
             created.
   */
  #define CREATE_SHM_OK               -1

  /*! \brief Return value for the function create_shared_memory_segment(). This means that the SHM has already been created.
   */
  #define CREATE_SHM_ALREADY_CREATED  -2

  /*! \brief Return value for the function create_shared_memory_segment(). This means that an error occured while creating
             the shared memory segment.
   */
  #define CREATE_SHM_ERROR            -3

  /* ============================================================= */
  /* Return value for the function get_shared_memory_segment_id()  */
  /* ============================================================= */

  /*! \brief Return value for the function get_shared_memory_segment_sys_id(). This means that the SHM system identifier
             has been looked up successfully.
   */
  #define GET_SHM_OK                  -1

  /*! \brief Return value for the function get_shared_memory_segment_sys_id(). This means that the SHM system identifier
             jas already been looked up.
   */
  #define GET_SHM_ALREADY_GET         -2

  /*! \brief Return value for the function get_shared_memory_segment_sys_id(). This means that an error occured while
             getting the system's identifier.
   */
  #define GET_SHM_ERROR               -3 

  /* ============================================================= */
  /* Module's API                                                  */
  /* ============================================================= */

  void change_shm_access_permission (int new_permission);
  int create_shared_memory_segment (key_t ipc_key, int size);
  int get_shared_memory_segment_sys_id (key_t ipc_key, int size);
  int get_shared_memory_segment_id();
  SHM_addr attache_shared_memory_segment ();
  SHM_addr get_shm_address();
  int set_shm_auto_destroy();
  struct shmid_ds* get_shm_info();
  int detache_shm();

  #define SHM_HD

#endif

#ifdef __cplusplus
}
#endif

