/*! \file my_sockets.h
    Header file for sockets.c.
 */

#ifdef __cplusplus
extern "C" {
#endif 


#ifndef SOCKETS_H

  #include <stdio.h>
  #include <stdlib.h>
  #include <unistd.h>
  #include <string.h>
  #include <sys/types.h>
  #include <sys/socket.h>
  #include <netdb.h>
  #include <netinet/in.h>
  #include <sys/param.h>
  #include <arpa/inet.h>


  int open_tcp_connexion (struct sockaddr_in*, char*, int, long, long, unsigned long int*);
  void Get_Sock_Adress (char *Machine_serveur, int port_serveur, struct sockaddr_in *TCP_address);
  int read_from_socket (int sockfd, char *buffer, size_t buffer_size, size_t* extracted, long tt_sec, long tt_micro, unsigned long int* ellapsed_time);
  int read_all_socket (int sockfd, char** buffer, long tt_sec, long tt_micro, unsigned long int* ellapsed_time, unsigned long int* size);
  void get_ip_addr(struct sockaddr_in *Address_Sender, char *IP_Address, int *Port);

  /* ----------------------------------------------------------- */
  /* Buffer sizes                                                */
  /* ----------------------------------------------------------- */

  /*! \brief Initial number of bytes for the buffer used to read data from a socket.
   */
  #define SCK_BUFFER_INIT_SIZE           1024

  /*! \brief Number of characters for an IP address.
   */

  #define SCK_ADDR_LEN                   16 

  /* ----------------------------------------------------------- */
  /* Error codes for open_tcp_connexion()                        */
  /* ----------------------------------------------------------- */

  /*! \brief Return value for the function open_tcp_connexion(). This means that the process can not create a new socket
             (in other words, the system call socket() failed).
   */
  #define SCK_CREATE_ERROR		-1

  /*! \brief Return value for the function open_tcp_connexion(). This means that the process can not connect to the remote
             server. In other words, the function connect() failed.
   */
  #define SCK_CONNECT_ERROR		-2

  /*! \brief Return value for the function open_tcp_connexion(). This means that the process can not get the IP address of
      the host running the remote server. In other words, the function gethostbyname() failed.
   */
  #define SCK_GET_IP_ERROR		-3

  /*! \brief Return value for the function open_tcp_connexion(). This means that the system call connect() has been
      interrupted by a signal (it should be SIGALRM.
   */  
  #define SCK_TIMEOUT_CONNECT		-4

  /*! \brief Return value for the function open_tcp_connexion(). This means that there was a problem with the high precision timer (that
             raises the signal SIGALRM). This error should not happen.
   */
  #define SCK_TIMER_ERROR               -5

  /*! \brief Return value for the function open_tcp_connexion(). This means that there was a problem with the high precision chronometer
   */
  #define SCK_CHRONO_ERROR              -6

  /* ----------------------------------------------------------- */
  /* Error codes fro read_all_socket() and read_from_socket()    */
  /* ----------------------------------------------------------- */

  /*! \brief Return value for the function read_all_socket() and read_from_socket(). This means that the function was successful.
   */
  #define SCK_READ_OK                   -7

  /*! \brief Return value for the function read_all_socket(). This means that the function can not allocate memory to store the data read
             from the socket.
   */
  #define SCK_READ_NO_MEM               -8

  /*! \brief Return value for the function read_all_socket() and read_from_socket(). This means that the function failed on the read()
             system call.
   */
  #define SCK_READ_READ_ERROR           -9

  /*! \brief Return value for the function read_all_socket() and read_from_socket(). Read timeout!
   */
  #define SCK_READ_TIMEOUT              -10

  /*! \brief Return value for the function read_all_socket() and read_from_socket(). This means that there was a problem with the high
             precision chronometer.
   */
  #define SCK_READ_CHRONO_ERROR         -11

  /*! \brief Return value for the function read_all_socket() and read_from_socket(). This means that there was a problem with the high
             precision timer.
   */
  #define SCK_READ_TIMER_ERROR          -12

  #define SOCKETS_H


#endif /* SOCKETS_H */


#ifdef __cplusplus
}
#endif
