/*! \file stack.h
    Header file for stack.c
 */

#ifndef STACK_H
#define STACK_H

#ifdef __cplusplus
extern "C" {
#endif

#include <stdlib.h>
#include "list.h"

/*! \brief A stack is implemented as a linked list.
 */

typedef List Stack;

/* ---------------------------------------------------------- */
/* Public API                                                 */
/* ---------------------------------------------------------- */

int stack_push(Stack *pile, const void *donnee);
int stack_pop(Stack *pile, void **donnee);

/*! \brief Initialize the stack. The prototype for this function is the same than for list_init().
           Please consult the documentation for the linked list API.
 */

#define stack_init list_init

/*! \brief Free all memory allocated for the stack. The prototype for this function is the same than for list_destroy().
           Please consult the documentation for the linked list API.
 */

#define stack_destroy list_destroy

/*! \brief Return a pointer to the element at the top of the list (the element is <B>NOT</B> extracted from the list).
    \param Pointer to a stack data structure.
 */

#define stack_peek(pile)\
        ((pile)->tete == NULL ? NULL : (pile)->tete->donnee)

/*! \brief Return the numder of element in the stack. This macro is identical to the macro <i>list_size</i> used for linked list.
           Please consult the documentation for the linked list API.
 */

#define stack_size list_size

#endif

#ifdef __cplusplus
}
#endif

