/*! \file strings_utils.h
    Header file for strings_utils.c
 */

#ifdef __cplusplus
extern "C" {
#endif



#ifndef STRINGS_UTILS_HPP

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>

/*! \brief This defines the minimum number of fields to allocate. 10 should ne enough.
 */

#define INITIAL_NUMBER_OF_FIELD 10

/*! \brief When a string is cut into fields, the fields are saved into a 'fields' structure.
 */

struct fields {
                /*! \brief Number of field extracted from the string. */
                int  number_of_fields;
                /*! \brief Number of allocated string. Please note that you can have 10 allocated fields, but only 7 are
                           actually used to put fields */
                int  size;
                /*! The field tabs points to an array of strings. Each string contains one field. */
                char **tabs;
              };

char *make_one_line (char *st);
int s_split(char *st, char* delimitor, struct fields *f);
int s_split_exact(char *st, char* delimitor, struct fields *f);
void free_fields (struct fields *fd);
char* get_end_of_string (char *src, char *dst, int nbcar);
char* IP2hex (char *src, char *dst, int nbcar);
char* post_chop(char *src);
char* pre_chop(char *src);
char* chop(char *src);
char* add_comment (char *src, char **dst);
char* get_config_value (char *buffer, char *delim);
char* replace_tag_by_integer (char* src, char *tag, int value);
unsigned int string2unsigned_int (char *st, int *error);
int string2int (char *st, int *error);
time_t string2time_t (char *st, int *error);
int is_integer (char *st);
int is_hexa (char *st);
int is_ip_address (char *st);
int ip_address_to_num (char *st, unsigned char *num);
int bin2hex (char *src, char *dst, int dst_size);
int bin2hex_binary (unsigned char *src, unsigned char *dst, int src_size, int dst_size);

#define STRINGS_UTILS_HPP
#endif


#ifdef __cplusplus
}
#endif

