/*! \file timing.h
    Header file for timing.c
 */

#ifdef __cplusplus
extern "C" {
#endif

#ifndef TIMING_HD

#include <sys/time.h>

  int start_chrono();
  int stop_chrono();
  unsigned long int get_ms(); 
  int set_one_shot_timer(long sec, long micro_second);
  int cancel_one_shot_timer();
  int pause_one_shot_timer();
  int restart_one_shot_timer();


/*! \brief Return value for the function set_one_shot_timer(). An error occured while setting the one shot timer.
 */
#define SET_ONE_SHOT_TIMER_ERROR        -1

/*! \brief Return value for the function set_one_shot_timer(). The one shot timer was successfully created.
 */
#define SET_ONE_SHOT_TIMER_OK           -2

/*! \brief Return value for the function cancel_one_shot_timer(). An error occured while canceling the one shot timer.
 */
#define CANCEL_ONE_SHOT_TIMER_ERROR     -3

/*! \brief Return value for the function cancel_one_shot_timer().  The one shot timer was successfully canceled.
 */
#define CANCEL_ONE_SHOT_TIMER_OK        -4

/*! \brief Return value for the function pause_one_shot_timer(). An error occured with the high precision timer.
 */
#define PAUSE_ONE_SHOT_TIMER_ERROR      -5

/*! \brief Return value for the function pause_one_shot_timer(). The one shot timer was successfully paused.
 */
#define PAUSE_ONE_SHOT_TIMER_OK         -6

/*! \brief Return value for the function restart_one_shot_timer(). An error occured with the high precision timer.
 */
#define RESTART_ONE_SHOT_TIMER_ERROR    -7

/*! \brief Return value for the function restart_one_shot_timer(). The one shot timer was successfully restarted.
 */
#define RESTART_ONE_SHOT_TIMER_OK       -8


#define TIMING_HD
#endif

#ifdef __cplusplus
}
#endif

