#!/opt/portal/6.0/bin/perl

#################################################################
#
#      ppp.pl
#      parse profil principal
#
# Migration LibertySurf
# Xavier Angles
#
# Version 1.0 11/09/2000
#
# NB: le numero de version principale est en coherence avec la
#     la version de format intermediaire
#
#################################################################

package Base64;
use Date::Parse;
use Time::Local;
require Commun_migration::codes_d_erreur_SI;
require Commun_migration::fonctions_de_controle;
require conf::conf_file;
require conf::file_list;
require Base64;

#
# Sous-programme de conversion ASCII -> UTF8
#
sub CAP_CONVERT_ASCII_TO_UTF8($)
{
  ($_)=@_;
  $_ =~ s/([\x80-\xFF])/chr(ord($1)>>6|192).chr(ord($1)&191)/ge;
  return $_;
}

#
# Declaration des variables du script
#
my @set_datelog;
my @set_cycle_size;
my @set_verbose;
my @set_newlog;
my @set_reprise;
my @set_nom_domaine;
my @lot;
my @c;
my @a;
my @multi_x;
my @operation_courante;
my @now;
my @profil_radius;
my @PROFIL_RADIUS;
my @profil_principal;
my @PROFIL_PRINCIPAL;
my @profil_civilite;
my @PROFIL_CIVILITE;
my @MARKETING_REF;
my @marketing_ref;
my @rejet_radius;
my @REJET_RADIUS;
my @rejet_principal;
my @REJET_PRINCIPAL;

#
# Declaration des variables applicatives
#
my @date_account;
my @ilogin;
my @codepers;
my @p3login;
my @codep3;
my @popname;
my @popnum;
my @home;
my @signup;
my @codemktg;
my @status;
my @idclient;
my @civilite;
my @nom;
my @prenom;
my @dnaissance;
my @adresse_1;
my @adresse_2;
my @adresse_3;
my @adresse;
my @code_postal;
my @ville;
my @email;

#
# Initialisation des variables
#
$poid_db=1;
$account_class="/account";
$poid_rev=0;
$actno_prefix="0.0.0.".$poid_db."-";
$service_ip_class="/service/ip/lsurf";
$set_cycle_size=5000;
$set_verbose=0;
$set_newlog=0;
$set_reprise=0;
$lot="00000";
$c=0;
$profil_principal="";
$rejet_principal="";
$account_poid="_account_poid.dat";
$domaine_de_migration="domaine_de_migration.txt";
$acc_no_premier_lot="new_seq_val_lot_premier.lst";
$acc_no_lot_courant="new_seq_val_lot_courant.lst";
$date_courante=localtime();
$now=str2time($date_courante);

#
# Sous-programme de log optionnel de la date
#
sub date_pour_log {
  my @date_locale;
  my @premiere_partie;
  my @seconde_partie;
  my %le_mois = (
'jan', '01',
'feb', '02',
'fev', '02',
'mar', '03',
'apr', '04',
'avr', '05',
'may', '06',
'mai', '06',
'jun', '07',
'jul', '07',
'jui', '07',
'aug', '08',
'aou', '08',
'sep', '09',
'oct', '10',
'nov', '11',
'dec', '12',
);
  if ($set_datelog)
    {
      $datelocale=localtime();
      chomp($datelocale);
      ($premiere_partie,$minute_locale,$seconde_partie) = split /:/, $datelocale;
      ($weekday_locale,$month_locale,$day_locale,$hour_locale) = split /\s/, $premiere_partie;
      ($second_locale,$year_locale) = split /\s/, $seconde_partie;
      $month_locale =~ tr/A-Z/a-z/;
      $month_locale =~ tr//eou/;
      $month_locale = $le_mois{substr($month_locale,0,3)};
      $datelocale= sprintf("%18s",$day_locale."/".$month_locale."/".$year_locale.$hour_locale.":".$minute_locale.":".$second_locale);
      return $datelocale;
    }
  else
    {
      return "                  ";
    }
}

#
# Traitement des arguments d'appel
#
$noms_de_fichiers=1;
foreach (@ARGV)
  {
    if ($_ eq "-datelog")
      {
        $set_datelog=1;
      }
    elsif ($_ eq "-verbose")
      {
        $set_verbose=1;
      }
    elsif ($_ eq "-reprise")
      {
        $set_reprise=1;
      }
    elsif ($_ eq "-newlog")
      {
        $set_newlog=1;
      }
    elsif (substr($_,0,2) eq "-c")
      {
        $set_cycle_size=substr $_,2;
        if ($set_cycle_size < 1)
          {
            print "\nLe cycle doit etre superieur a 1 comptes\n";
            exit (-1);
          }
      }
    elsif (substr($_,0,2) eq "-l")
      {
        $lot=substr $_,2;
        if ($lot < 0 or $lot > 99999)
          {
            print "\nLe lot de depart doit etre compris entre 0 et 99999\n";
            exit (-1);
          }
        else
          {
            $lot=sprintf("%05d",$lot);
          }
      }
    elsif ($_ eq "-help" || $_ eq "-\?")
      {
        print "\n";
        print "Usage: ppp.pl [-datelog] [-cn] [-ln] [-reprise] [-help|\?] [-newlog] [-verbose] pp.dat \n\n";
        print "\n";
        print "-c: nombre de comptes par lot, minimum 1000, par defaut ";
        print $set_cycle_size;
        print "\n";
        print "-l: lot de depart, maximum 99999, par defaut 0\n";
        print "-dalelog: memorise la date dans le fichier de log (plus lent)\n";
        print "\n";
        exit (0);
      }
    else
      {
        if ($noms_de_fichiers==1)
          {
            $profil_principal=$_;
            $noms_de_fichiers++;
            if (length $profil_principal < 5)
              {
                print "\nLe nom du fichier, $profil_principal n'est pas correct.\n";
                exit (-1);
              }
            elsif (substr($profil_principal,(length $profil_principal)-4,4) ne ".dat")
              {
                print "\nL'extension du fichier $profil_principal doit etre .dat\n";
                exit (-1);
              }
          }
      }
  }

#
# Initialisation du fichier log
#
$log=$profil_principal;
$log=~s/.dat$/.log/;

#
# Remise a zero du fichier log en mode -newlog
#
if ($set_newlog)
  {
    open LOG,">log/$log" or die "Impossible d'initialiser le fichier $log: $!\n";
    close LOG;
  }

#
# Ouverture du fichier de log
#
open LOG,">>log/$log" or die "Impossible d'ouvrir le fichier $log: $!\n";
print LOG "\n";


#
# Lecture du profil principal
#
$operation_courante="op01";

if ( $set_reprise )
  {
    #
    # En cas de reprise d'une execution precedente, on recupere les informations
    # sur le dernier lot en cours de traitement au moment de l'arret
    #
    open ACC_NO_LOT_COURANT,"conf/".$acc_no_lot_courant or die "Impossible d'ouvrir le fichier $acc_no_lot_courant: $!\n";
    $lot=(<ACC_NO_LOT_COURANT>);
    chomp($lot);
    if ($lot eq "TERMINE")
      {
        print "ERREUR: le traitement precedent a ete termine\n";
        exit (-1);
      }
    else
      {
        $account_no=(<ACC_NO_LOT_COURANT>);
        chomp($account_no);
        $set_cycle_size=(<ACC_NO_LOT_COURANT>);
        chomp($set_cycle_size);
        close ACC_NO_LOT_COURANT;
        $c=$lot*$set_cycle_size;
        $lot=sprintf("%05d",$lot);
        print "Le lot en cours etait le ";
        print $lot;
        print ", premier compte ";
        print $account_no;
        print ", cycle de ";
        print $set_cycle_size;
        print " comptes\n";

        #
        # Ecriture du log
        #
        print LOG "                                                                ";
        print LOG $pas_d_erreur;
        print LOG &date_pour_log;
        print LOG "                                                                ";
        print LOG $operation_courante;
        print LOG " - reprise du lot ";
        print LOG $lot;
        print LOG ", premier compte ";
        print LOG $account_no;
        print LOG ", cycle de ";
        print LOG $set_cycle_size;
        print LOG " comptes";
        print LOG "\n";

      }
  }
else
  {
    #
    # Recuperation du premier poid a creer
    #
    open PROFIL_PRINCIPAL,"data/".$profil_principal or die "Impossible d'ouvrir le fichier $profil_principal: $!\n";
    $line=<PROFIL_PRINCIPAL>;
    chomp($line);
    $access_code1=substr $line, 0, 80;
    $access_code1=~s/^ +//;
    $access_code1=~s/ +$//;
    $account_no=$poid_account{$access_code1};
    close PROFIL_PRINCIPAL;

    #
    # Ecriture du log
    #
    print LOG "                                                                ";
    print LOG "000";
    print LOG &date_pour_log;
    print LOG "                                                                ";
    print LOG $operation_courante;
    print LOG " - premier POID de compte impact ";
    print LOG $account_no;
    print LOG "\n";
  }

#
# Ouverture du fichier d'entree
#
open PROFIL_PRINCIPAL,"data/".$profil_principal or die "Impossible d'ouvrir le fichier $profil_principal: $!\n";

#
# Ecriture du log
#
print LOG "                                                                ";
print LOG $pas_d_erreur;
print LOG &date_pour_log;
print LOG "                                                                ";
print LOG $operation_courante;
print LOG " - ouverture en lecture de ";
print LOG $profil_principal;
print LOG "\n";


#
# Memorisation du premier POID utilise et de la taille du cycle
#
open ACC_NO_PREMIER_LOT,">conf/$acc_no_premier_lot" or die "Impossible d'ouvrir le fichier $acc_no_premier_lot: $!\n";
print ACC_NO_PREMIER_LOT $account_no;
print ACC_NO_PREMIER_LOT "\n";
print ACC_NO_PREMIER_LOT $set_cycle_size;
print ACC_NO_PREMIER_LOT "\n";
close ACC_NO_PREMIER_LOT;

#
# Ecriture du log
#
print LOG "                                                                ";
print LOG $pas_d_erreur;
print LOG &date_pour_log;
print LOG "                                                                ";
print LOG $operation_courante;
print LOG " - ecriture de ";
print LOG $acc_no_premier_lot;
print LOG "\n";

#
# En cas de reprise, on avance dans le fichier jusqu'au premier compte a traiter
#
if ($set_reprise)
  {
    $multi_x=$c;
    print "reprise a la ligne ";
    print $c;
    print " ...\n";
    while (<PROFIL_PRINCIPAL>)
      {
        $ligne=$_;
        chomp($ligne);
        if (&controle_ligne_pp eq $pas_d_erreur)
          {
            $c-=1;
          }
        last if $c==0;
      }

    #
    # Ecriture du log
    #
    print LOG "                                                                ";
    print LOG $pas_d_erreur;
    print LOG &date_pour_log;
    print LOG "                                                                ";
    print LOG $operation_courante;
    print LOG " - avance jusqu'a ";
    print LOG $multi_x;
    print LOG "\n";
  }

#
# Ouverture du fichier de rejets principal
#
$rejet_principal=$profil_principal;
$rejet_principal=~s/.dat$/.bad/;
open REJET_PRINCIPAL,">>rejets/".$rejet_principal or die "Impossible d'ouvrir le fichier $rejet_principal: $!\n";

#
# Ecriture du initialisation du fichier de rejets
#
print REJET_PRINCIPAL "                                                                ";
print REJET_PRINCIPAL $pas_d_erreur;
print REJET_PRINCIPAL &date_pour_log;
print REJET_PRINCIPAL "                                                                ";
print REJET_PRINCIPAL $operation_courante;
print REJET_PRINCIPAL " - ouverture en lecture de ";
print REJET_PRINCIPAL $profil_principal;
print REJET_PRINCIPAL "\n";


#
# Debut du traitement
#
$operation_courante="op10";

#
# Initialisation du nom du fichier de sortie
#
$profil_principal_sql=$profil_principal;
$profil_principal_sql=~s/.dat//;
$profil_principal_sql=$profil_principal_sql."_infranet_".$operation_courante."_".$lot.".nap";

#
# Ouverture du fichier de sortie
#
open PROFIL_PRINCIPAL_NAP,">nap/$profil_principal_sql" or die "Impossible d'ouvrir le fichier $profil_principal_sql: $!\n";

#
# Ecriture du log
#
print LOG "                                                                ";
print LOG $pas_d_erreur;
print LOG &date_pour_log;
print LOG "                                                                ";
print LOG $operation_courante;
print LOG " - ouverture en ecriture de ";
print LOG $profil_principal_sql;
print LOG "\n";

print $profil_principal_sql;
print "\n";

$num_ligne=0;
while (<PROFIL_PRINCIPAL>)
  {
    $ligne=$_;
    chomp($ligne);
    $num_ligne++;


        $c++;


        if ($c==($set_cycle_size+1))
          {
            #
            # Le nombre de compte du lot a atteint le nombre requis
            #
            print PROFIL_PRINCIPAL_NAP "q\n\n";


            close PROFIL_PRINCIPAL_NAP;

            #
            # On passe au lot suivant
            #
            $lot++;

            #
            # Initialisation du fichier de sortie
            #
            $profil_principal_sql=$profil_principal;
            $profil_principal_sql=~s/.dat//;
            $profil_principal_sql=$profil_principal_sql."_infranet_".$operation_courante."_".$lot.".nap";

            #
            # Traces temporaires
            #
            #print "Nom du fichier de sortie: ";
            print $profil_principal_sql;
            print "\n";
            #print "taper RETURN pour continuer ...";
            #$multi_x=<stdin>;

            #
            # Ouverture du fichier de sortie
            #
            open PROFIL_PRINCIPAL_NAP,">nap/$profil_principal_sql" or die "Impossible d'ouvrir le fichier $profil_principal_sql: $!\n";

            $c=1;
          }


        #
        # Recuperation des donnees dans le fichier d'entree
        #
        $poid_line = substr $ligne, 0;
        #@chaine = split /;/, $poid_line;
        ($end_date, $User_Name, $NAS_IP_Address, $NAS_Port, $NAS_Port_Type, $Class, $Acct_Status_Type, $Acct_Delay_Time, $Acct_Session_Id, $Acct_Authentic, $Acct_Session_Time, $Acct_Input_Octets, $Acct_Output_Octets, $Acct_Input_Packets, $Acct_Output_Packets, $Disconnect_Cause, $Connect_Progress, $Xmit_Rate, $Data_Rate, $PreSession_Time, $Pre_Input_Octets, $Pre_Output_Octets, $Pre_Input_Packets, $Pre_Output_Packets, $First_Dest, $Modem_PortNo, $Modem_SlotNo, $Modem_ShelfNo, $Calling_Station_Id, $Called_Station_Id, $Framed_Protocol, $Framed_IP_Address, $Pseudo_Request_Source, $Pseudo_Request_Type)  = split /;/, $poid_line;

        #$end_date=@chaine[0];
	     #$user-name=@chaine[1];
        #$NAS-IP-Address=@chaine[2];	
        #$NAS-Port=@chaine[3];
        #$Acct-Status-Type=@chaine[6];
        #$Acct-Delay-Time=@chaine[7];
        #$Acct-Session-Id=@chaine[8];
        #$Acct-Session-Time=@chaine[10];
        #$Acct-Input-Octets=@chaine[11];
        #$Acct-Output-Octets=@chaine[12];
        #$Acct-Input-Packets=@chaine[13];
        #$Acct-Output-Packets=@chaine[14];
        #$Disconnect-Cause=@chaine[15];
        #$Data-Rate=@chaine[18];
        #$Calling-Station-Id=@chaine[28];
        #$Called-Station-Idd=@chaine[29];
        #$Framed-IP-Address=@chaine[31];
        

        #
        # Traces optionnelles d'execution en mode -verbose
        #
        if ($set_verbose)
          {
            print "id:";
            print $access_code1;
            print " POID:";
            print $account_no;
            print " ligne:";
            print $.;
            print " lot:";
            print $lot;
            print " rang:";
            print $c;
            print "\n";
          }

        if ($c==1||$init==1)
          {
            #
            # Memorisation du numero de lot, du premier POID et du premier ilogin
            # du lot courant
            #
            open ACC_NO_LOT_COURANT,">conf/$acc_no_lot_courant" or die "Impossible d'ouvrir le fichier $acc_no_lot_courant: $!\n";
            print ACC_NO_LOT_COURANT $lot;
            print ACC_NO_LOT_COURANT "\n";
            print ACC_NO_LOT_COURANT $account_no;
            print ACC_NO_LOT_COURANT "\n";
            print ACC_NO_LOT_COURANT $set_cycle_size;
            print ACC_NO_LOT_COURANT "\n";
            close ACC_NO_LOT_COURANT;

            #
            # Ecriture du log
            #
            print LOG sprintf("%80s",$access_code1);
            print LOG $pas_d_erreur;
            print LOG &date_pour_log;
            print LOG "                                                                ";
            print LOG $operation_courante;
            print LOG " - ecriture de ";
            print LOG $acc_no_lot_courant;
            print LOG "\n";

            print PROFIL_PRINCIPAL_NAP "p logging on\np display_results off\np\n";

            $init=0;
          }

        
        $login_utf8=CAP_CONVERT_ASCII_TO_UTF8($User_Name);
        $login_utf8=~s/"/\\"/g;
        if (length $Framed_IP_Address != 8)
           {
           print "\n Erreur Format IP\n";
           exit; 
           }
        



        #
        # Ecriture de la ligne dans le fichier de sortie
        #
        print PROFIL_PRINCIPAL_NAP "r << XXX 1\n";
        print PROFIL_PRINCIPAL_NAP "0 PIN_FLD_POID POID [0] 0.0.0.1 /service/ip/ltelecom -1 0\n";
        print PROFIL_PRINCIPAL_NAP "0 PIN_FLD_LOGIN STR [0] \"$login_utf8\"\n";
        print PROFIL_PRINCIPAL_NAP "0 PIN_FLD_TERMSERV_ID STR [0] \"$NAS_IP_Address\"\n";
        print PROFIL_PRINCIPAL_NAP "0 PIN_FLD_TERMSERV_PORT  STR [0] \"$NAS_Port\"\n";
        print PROFIL_PRINCIPAL_NAP "0 PIN_FLD_TRANS_ID  STR [0] \"$Acct_Session_Id\"\n";
        print PROFIL_PRINCIPAL_NAP "0 PIN_FLD_PROGRAM_NAME STR [0] \"Migration Event Loader\"\n";
        print PROFIL_PRINCIPAL_NAP "0 PIN_FLD_STATUS  ENUM [0] $Disconnect_Cause \n";
        print PROFIL_PRINCIPAL_NAP "0 PIN_FLD_BYTES_IN INT [0] $Acct_Input_Octets  \n";
        print PROFIL_PRINCIPAL_NAP "0 PIN_FLD_BYTES_OUT INT [0] $Acct_Output_Octets  \n";
        print PROFIL_PRINCIPAL_NAP "0 PIN_FLD_PACKETS_IN INT [0] $Acct_Input_Packets  \n";
        print PROFIL_PRINCIPAL_NAP "0 PIN_FLD_PACKETS_OUT INT [0] $Acct_Output_Packets  \n";
        print PROFIL_PRINCIPAL_NAP "0 PIN_FLD_DELAY_TIME INT [0] $Acct_Delay_Time \n";
        print PROFIL_PRINCIPAL_NAP "0 PIN_FLD_ANI  STR [0] \"$Calling_Station_Id\"\n";
        print PROFIL_PRINCIPAL_NAP "0 PIN_FLD_DNIS STR [0] \"$Called_Station_Id\"\n";
        print PROFIL_PRINCIPAL_NAP "0 PIN_FLD_IPADDR  BINSTR [0] 4 $Framed_IP_Address\n";
        print PROFIL_PRINCIPAL_NAP "0 PIN_FLD_SPEED INT [0] $Data_Rate\n";
        print PROFIL_PRINCIPAL_NAP "0 PIN_FLD_TYPE_STR STR [0] \"$Acct_Status_Type\"\n";
        print PROFIL_PRINCIPAL_NAP "0 PIN_FLD_END_T TSTAMP [0] ($end_date)\n";
        print PROFIL_PRINCIPAL_NAP "0 PIN_FLD_ELAPSED_TIME TSTAMP [0] ($Acct_Session_Time)\n";
 
        print PROFIL_PRINCIPAL_NAP "XXX\n";

        print PROFIL_PRINCIPAL_NAP "d 1\n";
        print PROFIL_PRINCIPAL_NAP "xop 357 0 1";
        print PROFIL_PRINCIPAL_NAP "\n";

        print PROFIL_PRINCIPAL_NAP "###\n\n";



  }

#
# On commit l'ensemble du lot
#
print PROFIL_PRINCIPAL_NAP "q\n\n";

#
# Fermeture des fichiers
#
close PROFIL_PRINCIPAL;
close PROFIL_PRINCIPAL_NAP;




# Terminaison du fichier de rejets
#
print REJET_PRINCIPAL "                                                                ";
print REJET_PRINCIPAL $pas_d_erreur;
print REJET_PRINCIPAL &date_pour_log;
print REJET_PRINCIPAL "                                                                ";
print REJET_PRINCIPAL $operation_courante;
print REJET_PRINCIPAL " - fermeture de ";
print REJET_PRINCIPAL $profil_principal;
print REJET_PRINCIPAL "\n";

close REJET_PRINCIPAL;

#
# Terminaison des fichiers de reprise
#
open ACC_NO_PREMIER_LOT,">conf/$acc_no_premier_lot" or die "Impossible d'ouvrir le fichier $acc_no_premier_lot: $!\n";
print ACC_NO_PREMIER_LOT "TERMINE\n";
close ACC_NO_PREMIER_LOT;
open ACC_NO_LOT_COURANT,">conf/$acc_no_lot_courant" or die "Impossible d'ouvrir le fichier $acc_no_lot_courant: $!\n";
print ACC_NO_LOT_COURANT "TERMINE\n";
close ACC_NO_LOT_COURANT;

#
# Ecriture du log
#
print LOG "                                                                ";
print LOG $pas_d_erreur;
print LOG &date_pour_log;
print LOG "                                                                ";
print LOG $operation_courante;
print LOG " - terminaison";
print LOG "\n";

close LOG;

