%{

#include <stdio.h>
#include <string.h>
#include "date.h"
#include "strings_utils.h"

extern int             date_line_num;
extern struct tz_tab   tzs;


static int             tz_num = 0;
static int             n;
static struct fields   fd; 


int dateyywrap() { return 1; }


%}

ALPHA         ([A-Z]|[a-z])
NUMERIC       [0-9]
ALPHANUM      ({ALPHA}|{NUMERIC})
SPACE         \x20
TABUL         \t
BLANK         [\x20\t]
NUMBER        [0-9]
INTEGER       {NUMBER}+
BLANK_LINE    ^{BLANK}+\n
COMMENT       ^{BLANK}*#.+\n
EMPTY_LINE    ^\n

TIMEZONE      {ALPHA}+
DAYLIGHT      {ALPHA}+
SHIFT         (\+{INTEGER})|(\-{INTEGER})

TIMING1       ^{TIMEZONE}{SPACE}+{SHIFT}{SPACE}*$
TIMING2       ^{TIMEZONE}{SPACE}+{DAYLIGHT}{SPACE}+{SHIFT}{SPACE}*$


%%

{TIMING1}     {
                if (tz_num >= MAX_TZ_DEF) { return TIME_ZONES_DEF_OVERFLOW; }

                n = s_split(yytext, " ", &fd);
                if (n == -1) { return TIME_ZONES_NO_MEM; }

                if (strlen(fd.tabs[0]) >= TIMEZONE_SIZE) { return TIME_ZONES_TIMEZONE_OVERFLOW; }
                strcpy (((tzs.tz)[tz_num]).timezone, fd.tabs[0]);
                ((tzs.tz)[tz_num].daylight)[0] = 0;
                ((tzs.tz)[tz_num]).shift = atoi(fd.tabs[1]);

                tz_num += 1;
                tzs.entry_number = tz_num;
                free_fields(&fd);
              }



{TIMING2}     {
                if (tz_num >= MAX_TZ_DEF) { return TIME_ZONES_DEF_OVERFLOW; }

                n = s_split(yytext, " ", &fd);
                if (n == -1) { return TIME_ZONES_NO_MEM; }

                if (strlen(fd.tabs[0]) >= TIMEZONE_SIZE) { return TIME_ZONES_TIMEZONE_OVERFLOW; }
                if (strlen(fd.tabs[1]) >= TIMEZONE_SIZE) { return TIME_ZONES_DAYLIGHT_OVERFLOW; }
                strcpy (((tzs.tz)[tz_num]).timezone, fd.tabs[0]);
                strcpy (((tzs.tz)[tz_num]).daylight, fd.tabs[1]);
                ((tzs.tz)[tz_num]).shift = atoi(fd.tabs[2]);

                tz_num += 1;
                tzs.entry_number = tz_num;
                free_fields(&fd);
              }



{BLANK_LINE} { date_line_num++; }
{EMPTY_LINE} { date_line_num++; }
{COMMENT}    { date_line_num++; }
\n           { date_line_num++; }
.*           { return TIME_ZONES_PARSE_ERROR; }

%%



