/*! \file deamon.c
    Simple interface used to create a UNIX daemon.
 */

#include "deamon.h"

/*! \brief Turn the calling process into a deamon.
    \param pid Pointer to a PID descriptor that will receive the PID returned by the call to fork().
               <UL>
                  <LI>If pid < 0, then an error occured while calling fork().
                  <LI>If pid > 0, then we are in the 'father' process and PID represents the child's PID.
                  <LI>If PID == 0, then we are in the 'child' process.
               </UL>
    \param wd Pointer to a zero terminated string of characters that represents the deamon's working directory.
    \return Upon successful completion, the function returns the value 0. otherwize, the function returns
            the value 1. 
 */

int deamonizer(pid_t *pid, char *wd)
{
        pid_t sid;

        *pid = (pid_t)-1;

        /* ------------------------------------- */ 
        /* Fork off the parent process           */
        /*  - pid < 0: error                     */
        /*  - pid > 0: we are in the father      */
        /* ------------------------------------- */ 

        *pid = fork();
        if (*pid < 0) { return 1; }
        if (*pid > 0) { return 0; }
       
        /* ------------------------------------- */ 
        /* Change the file mode mask             */
        /* ------------------------------------- */ 

        umask(0);       
        
        /* ------------------------------------- */ 
        /* Create a new SID for the child        */
        /* process                               */ 
        /* ------------------------------------- */ 

        sid = setsid();
        if (sid < 0) { return 1; }
        
        /* ------------------------------------- */ 
        /* Change the current working directory  */
        /* ------------------------------------- */ 

        if ((chdir(wd)) < 0) { return 1; }
        
        /* ------------------------------------- */ 
        /* Close out the standard file           */
        /* Descriptors                           */
        /* ------------------------------------- */ 

        close(STDIN_FILENO);
        close(STDOUT_FILENO);
        close(STDERR_FILENO);

        return 0;
}

