/*! \file hashfunc.c
    This file contains various hash functions.
 */

#include "hashfunc.h"


/*! \brief Calculate an index from a string of charecters.
    \param Key Pointer to a zero terminated string of characters.
    \param TableSize 'TableSize-1' represents the maximum value possible for the index. In other words, the value of 
           the returned index will be such as:  0 &lt;= index &lt;= (TableSize-1).
 */

Index Hash1( const char *Key, int TableSize )
{
  unsigned int HashVal = 0;

  while(*Key != '\0') { HashVal += *Key++; }
  HashVal += *Key++;

  return HashVal % TableSize;
}

/*! \example test_hash_func.c
    This file shows how to use the hashing functions.
 */

/*! \brief Calculate an index from a string of charecters.
    \param Key Pointer to a zero terminated string of characters.
    \param TableSize 'TableSize-1' represents the maximum value possible for the index. In other words, the value of
           the returned index will be such as:  0 &lt;= index &lt;= (TableSize-1).
 */

Index Hash2( const char *Key, int TableSize )
{ return ( Key[ 0 ] + 27 * Key[ 1 ] + 729 * Key[ 2 ] ) % TableSize; }

/*! \example test_hash_func.c
    This file shows how to use the hashing functions.
 */

/*! \brief Calculate an index from a string of charecters.
    \param Key Pointer to a zero terminated string of characters.
    \param TableSize 'TableSize-1' represents the maximum value possible for the index. In other words, the value of
           the returned index will be such as:  0 &lt;= index &lt;= (TableSize-1).
 */

Index Hash3( const char *Key, int TableSize )
{
  unsigned int HashVal = 0;

  while( *Key != '\0' ) { HashVal = ( HashVal << 5 ) + *Key++; }

  return HashVal % TableSize;
}

/*! \example test_hash_func.c
    This file shows how to use the hashing functions.
 */

