/*! \file my_math.c
    This file implements various specific mathematical tools.
 */

#include "my_math.h"

/*! \brief Given two integers <i>N</i> and <i>p</i>, the function returns the integer <i>r</i> such as:<BR>
           <UL>
              <li><i>r</i> is a multiple of <i>p</i>.
              <li>&quot;<i>r</i> = <i>N</i> + <i>x</i>&quot; with <i>x</i> such as &quot;0 &lt;= <i>x</i> &lt; <i>N</i>&quot;
           </UL>
           For example, if <i>N</i>=2 and <i>p</i>=4, then <i>r</i>=4.
    \param value This parameter matches <i>N</i> in the function's description above.
    \param factor This parameter matches <i>p</i> in the function's description above.
    \return The function returns the multiple of <i>factor</i>, that is closest to <i>value</i> (greater that <i>value</i>).
 */

unsigned int round_to_multiple_of (unsigned int value, unsigned int factor)
{
    unsigned int d;

    d = value / factor;

    if ((d * factor) != value) { d++; }
    return d * factor;
}






