%{

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "list.h"
#include "dstring.h"
#include "replacer.h"

/* -------------------------------------------------- */
/* this file contains the definitions of a 'tag' for  */
/* the Search / Replace facility.                     */
/* -------------------------------------------------- */


int hashwrap() { return 1; }

extern int hashlex();

#define KEEP_GOING       0
#define CREATE_NEW       1

static int element; /* KEEP_GOING / CREATE_NEW */

/* -------------------------------------------------- */
/* Linked list management                             */
/* We define a linked list of dynamic strings.        */
/*                                                    */
/* o hash_linked_list: the linked list.               */
/* o free_data: function used to free one element of  */
/*   the linked list.                                 */
/* o compare_element: function used to compare two    */
/*   elements of the linked list (not used).          */
/* o init_hash_linked_list: Initialize the linked     */
/*   list.                                            */
/* -------------------------------------------------- */

#define DSTRING_INI_SIZE 100

static List hash_linked_list;
static dstring *dds;
static char *buff;

static void free_data (void *d)
{
  dstring *ds;

  ds = (dstring*)d;

  dstring_free (ds);
  free (ds);
}

static int compare_element (const void* d1, const void* d2) { return 1; }

static void init_hash_linked_list ()
{ list_init (&hash_linked_list, free_data, compare_element); }





/* -------------------------------------------------- */
/* This function calls the flex parser to parse       */
/* in-memory text.                                    */
/* -------------------------------------------------- */

char **txt_tokens;  /* the result itself, made from the likned list */
size_t nb_tokens;   /* the number of elements in the result         */

void free_tokens()
{
  size_t i;

  if (txt_tokens != NULL)
  {
    for (i=0; i<nb_tokens; i++) { free(txt_tokens[i]); }
    free (txt_tokens);
  }
}

int yylex_memory (char *src)
{
  int             result;
  YY_BUFFER_STATE buf_state;
  size_t          i, s;
  ListElmt        *elem;

  /* ------------------------------------------------ */
  /* Initilaze global variables                       */
  /* ------------------------------------------------ */
  
  init_hash_linked_list();
  element    = CREATE_NEW;
  nb_tokens  = 0;
  txt_tokens = NULL;

  /* ------------------------------------------------ */
  /* Parse in-memory text and create the linked list  */
  /* ------------------------------------------------ */

  buf_state = yy_scan_string(src);
  result = hashlex();
  yy_delete_buffer(buf_state);
  if (result == RELACER_MEM_ERROR) { return RELACER_MEM_ERROR; }

  /* ------------------------------------------------ */
  /* Convert the linked list into an array of strings */
  /* ------------------------------------------------ */

  nb_tokens = list_size (&hash_linked_list);

  txt_tokens = (char**)malloc(nb_tokens*sizeof(char**));
  if (txt_tokens == NULL) 
  {
    list_destroy(&hash_linked_list);
    return RELACER_MEM_ERROR;
  }

  elem = list_head(&hash_linked_list);

  for (i=0; i<nb_tokens; i++)
  {
    dds = (dstring*) list_data(elem);
    txt_tokens[i] = dstring_get_data (dds, &s);
    if (txt_tokens[i] == NULL) { return RELACER_MEM_ERROR; }

    elem = list_next(elem);
  }

  /* ------------------------------------------------ */
  /* Free all memory allocated from the linked list   */
  /* ------------------------------------------------ */

  list_destroy(&hash_linked_list);

  return result;
}



/* ############################################################# */
/* ###                                                       ### */
/* ### WARNING !!!!                                          ### */
/* ###                                                       ### */
/* ### If you change the definition of the TAG, make sure to ### */
/* ### change the code of the flex action associated with a  ### */
/* ### tag detection.                                        ### */
/* ###                                                       ### */
/* ### You must also change the code of the file             ### */
/* ### 'replacer.c' (the section where you replace a tag by  ### */
/* ### its value.                                            ### */
/* ###                                                       ### */
/* ############################################################# */



%}

ALPHA         ([A-Z]|[a-z])
NUMERIC       [0-9]
ALPHANUM      ({ALPHA}|{NUMERIC})
CARACTER      ({ALPHANUM}|[_\-])
TAG           \$\{{CARACTER}+\}

%%

{TAG}   {
           /* -------------------------------------------------- */
           /* Allocate a new Dynamic string to store the tag.    */
           /* -------------------------------------------------- */

           dds = (dstring*)malloc(sizeof(dstring));
           if (dds == NULL) { return RELACER_MEM_ERROR; }

           /* -------------------------------------------------- */
           /* initialize the Dynamic String                      */
           /* -------------------------------------------------- */

           if (dstring_init (dds, DSTRING_INI_SIZE) == 1)
           { return RELACER_MEM_ERROR; }

           /* -------------------------------------------------- */
           /* Remove unwanted formating characters               */
           /* -------------------------------------------------- */

           buff = (char*)malloc(strlen(yytext)+1);
           if (buff == NULL) { return RELACER_MEM_ERROR; }

           strcpy (buff, yytext);
           buff[strlen(yytext)-1] = 0;

           /* -------------------------------------------------- */
           /* Add data into the Dynamic String                   */
           /*                                                    */
           /*     A tag begins with a +                          */
           /*                                                    */
           /* -------------------------------------------------- */

           if (dstring_add (dds, "+", 1) == 1)
           { free (buff); return RELACER_MEM_ERROR; }

           if (dstring_add (dds, buff+2, strlen(buff+2)) == 1)
           { free (buff); return RELACER_MEM_ERROR; }

           free (buff);

           /* -------------------------------------------------- */
           /* Add Dynamic String to linked list                  */
           /* -------------------------------------------------- */
           
           if (list_ins_next (&hash_linked_list, list_tail(&hash_linked_list), (void*)dds) == -1)
           { return RELACER_MEM_ERROR; }

           element = CREATE_NEW;
        }

(.|\n)  {
           if (element == CREATE_NEW)
           {
             /* -------------------------------------------------- */
             /* Allocate a new Dynamic string to store the tag.    */
             /* -------------------------------------------------- */

             dds = (dstring*)malloc(sizeof(dstring));
             if (dds == NULL) { return RELACER_MEM_ERROR; }

             /* -------------------------------------------------- */
             /* initialize the Dynamic String                      */
             /*                                                    */
             /*     A tag begins with a -                          */
             /*                                                    */
             /* -------------------------------------------------- */

             if (dstring_init (dds, DSTRING_INI_SIZE) == 1)
             { return RELACER_MEM_ERROR; }

             if (dstring_add (dds, "-", 1) == 1)
             { return RELACER_MEM_ERROR; }

             /* -------------------------------------------------- */
             /* Add Dynamic String to linked list                  */
             /* -------------------------------------------------- */

             if (list_ins_next (&hash_linked_list, list_tail(&hash_linked_list), (void*)dds) == -1)
             { return RELACER_MEM_ERROR; }
           }

           /* ---------------------------------------------------- */
           /* Add data to the last element of the linked list      */
           /* ---------------------------------------------------- */

           dds = (dstring*)list_data(list_tail(&hash_linked_list));

           if (dstring_add (dds, yytext, strlen(yytext)) == 1)
           { return RELACER_MEM_ERROR; }

           element = KEEP_GOING;
        }

%%

