/*! \file stack.c
    This file implements a basic stack API.
 */


#include <stdlib.h>

#include "list.h"
#include "stack.h"

/*! \brief Stack up data at the top of the stack.
    \param pile Pointer to a stack data structure.
    \param donnee Pointer to the data to stack up.
    \return Upon successful completion the function return the value 0. Otherwise the function returns -1.
    \warning The inserted data is <B>NOT</B> copied into the linked list! Therefore you should not free it unless you know exactly what you
             are doing.
 */

int stack_push(Stack *pile, const void *donnee) { return list_ins_next(pile, NULL, donnee); }

/*! \example test_stack.c
    This test prpgram shows how to use the stack API.
 */

/*! \brief Extract data from the top of the stack.
    \param pile Pointer to a stack data structure.
    \param donnee Pointer to a pointer the will point to the extracted data.
    \return Upon successful completion the function return the value 0. Otherwise the function returns -1.
    \warning This function does <B>NOT</B> free the element's data (it only frees the element's structure). Therefore you should use
             <i>donnee</i> to free the element's data yourself.
 */

int stack_pop(Stack *pile, void **donnee) { return list_rem_next(pile, NULL, donnee); }

/*! \example test_stack.c
    This test prpgram shows how to use the stack API.
 */



