%{

#include <stdio.h>
#include <string.h>
#include "date.h"
#include "strings_utils.h"

extern int                ttref_line_num;
extern struct ttref_tab   ttrefs;


static int                ttref_num = 0;
static int                n;
static struct fields      fd; 
static char*              text_p;


int ttrefwrap() { return 1; }


%}

ALPHA         ([A-Z]|[a-z])
NUMERIC       [0-9]
ALPHANUM      ({ALPHA}|{NUMERIC})
SPACE         \x20
TABUL         \t
BLANK         [\x20\t]
NUMBER        [0-9]
INTEGER       {NUMBER}+
BLANK_LINE    ^{BLANK}+\n
COMMENT       ^{BLANK}*#.+\n
EMPTY_LINE    ^\n

TIMEZONE      {ALPHA}+
DAYLIGHT      {ALPHA}+
SHIFT         (\+{INTEGER})|(\-{INTEGER})

REFERENCE     [^=#]+{BLANK}*={BLANK}*{INTEGER}{BLANK}*$

%%

{REFERENCE}   {
                text_p =  (char*)malloc((strlen(yytext)+1)*sizeof(char));
                if (text_p == NULL) { return TTREF_NO_MEM; }
                strcpy (text_p, yytext);

                if (ttref_num >= MAX_TTREF_ENTRIES) { free(text_p); return TTREF_TOO_MANY_REF; }
                n = s_split(text_p, "=", &fd);
                free(text_p);
                if (n == -1) { return TTREF_NO_MEM; }
                if (n != 2)  { return TTREF_SYNTAX_ERROR; }

                if (chop (fd.tabs[0]) == NULL) { return TTREF_NO_MEM; }
                if (chop (fd.tabs[1]) == NULL) { return TTREF_NO_MEM; }

                #ifdef DEBUG
                fprintf (stdout, "\n => [%s]", fd.tabs[0]);
                fprintf (stdout, "\n => [%s]", fd.tabs[1]);
                #endif

                if (strlen(fd.tabs[0]) >= TTREF_MAX_DATE_SIZE) { return TTREF_DATE_TOO_LONG; }
                strcpy (((ttrefs.ttref)[ttref_num]).date, fd.tabs[0]);

                if (strlen(fd.tabs[1]) >= TTREF_MAX_TIMESTAMP_SIZE) { return TTREF_REF_TOO_LONG; }
                strcpy (((ttrefs.ttref)[ttref_num]).timestamp, fd.tabs[1]);

                ttref_num += 1;
                ttrefs.entry_number = ttref_num;
                free_fields(&fd);
              }


{BLANK_LINE} { ttref_line_num++; }
{EMPTY_LINE} { ttref_line_num++; }
{COMMENT}    { ttref_line_num++; }
\n           { ttref_line_num++; }
.*           { return TTREF_SYNTAX_ERROR; }

%%



