#include <stdlib.h>
#include <stdio.h>
#include "double_list.h"



/* ----------------------------------------------------------- */
/* Type of data to put into the linked list                    */
/* ----------------------------------------------------------- */

struct sdata {
                int   size;
                char  *s;    /* dinamically allocated */
             };

/* ----------------------------------------------------------- */
/* Function used to destroy the data                           */
/* ----------------------------------------------------------- */

void free_data (void* d)
{
  struct sdata* data;
  data = (struct sdata*)d;

  free (data->s);
  free (data);

  return;
}

/* ----------------------------------------------------------- */
/* Function used to compare two elements                       */
/* ----------------------------------------------------------- */

int compare_element (const void* d1, const void* d2)
{
  struct sdata *data1, *data2;
  data1 = (struct sdata*)d1;
  data2 = (struct sdata*)d2;

  if (strcmp(data1->s, data2->s) == 0) { return 1; }

  return 0;
}

/* ----------------------------------------------------------- */
/* Program's entry point                                       */
/* ----------------------------------------------------------- */

int main (int argc, char *argv[])
{
   List          liste;
   int           i, max;
   struct sdata  *data, d;
   char          buff[32];
   ListElmt      *element, *prec;

   /* -------------------------------------------------------- */
   /* Check command line arguments                             */
   /* -------------------------------------------------------- */

   if (argc != 2)
   {
      fprintf (stderr, "\nUsage: list.test <number of elements>\n");
      return 1;
   }

   max = atoi (argv[1]);

   /* -------------------------------------------------------- */
   /* initialize the linked list                               */
   /* -------------------------------------------------------- */

   list_init (&liste, free_data, compare_element);

   /* -------------------------------------------------------- */
   /* Fill linked list with elements                           */
   /* -------------------------------------------------------- */

   fprintf (stdout, "\nFill the linked list with data\n");

   for (i=0; i<max; i++)
   {
     /* ------------------------------------------------------ */
     /* Allocate new data structure                            */
     /* ------------------------------------------------------ */

     data = (struct sdata*)malloc(sizeof(struct sdata));
     if (data == NULL)
     {
        fprintf (stderr, "\nError: can not allocate memory\n");
        return 1;
     }

     /* ------------------------------------------------------ */
     /* initialize the newly allocated data structure          */
     /* ------------------------------------------------------ */

     sprintf (buff, "%d", i);
     data->size = strlen(buff);

     data->s = (char*)malloc((data->size+1)*sizeof(char));
     if (data->s == NULL)
     {
        fprintf (stderr, "\nError: can not allocate memory\n");
        return 1;
     }

     strcpy (data->s, buff);

     /* ------------------------------------------------------ */
     /* Add a new element to the linked list                   */
     /* ------------------------------------------------------ */

     if (list_ins_next (&liste, list_tail(&liste), data) == -1)
     {
        fprintf (stderr, "\nError: can not allocate memory\n");
        return 1;
     }
   }

   /* -------------------------------------------------------- */
   /* Print the linked list from the start to the end          */
   /* -------------------------------------------------------- */

   fprintf (stdout, "\nPrint the content of the linked list from the start to the end\n");

   element = list_head (&liste);

   while (element != NULL)
   {
     fprintf (stdout, "\n%s", ((struct sdata*)list_data(element))->s);
     element = list_next(element);
   }

   /* -------------------------------------------------------- */
   /* Print the linked list from the end to the start          */
   /* -------------------------------------------------------- */

   fprintf (stdout, "\nPrint the content of the linked list from the end to the start\n");

   element = list_tail (&liste);

   while (element != NULL)
   {
     fprintf (stdout, "\n%s", ((struct sdata*)list_data(element))->s);
     element = list_prev(element);
   }

   /* -------------------------------------------------------- */
   /* Searching the linked list                                */
   /* -------------------------------------------------------- */

      /* ----------------------------------------------------- */
      /* First search                                          */
      /* ----------------------------------------------------- */

   d.size = 1;
   sprintf (buff, "4");
   d.s = buff;

   fprintf (stdout, "\nSearching (from the beginning) for element containing [%s]", d.s);
   element = list_find (&liste, NULL, &d, &prec);
   if (element == NULL)
   {
     fprintf (stdout, "\nNot found");
   }
   else
   {
     fprintf (stdout, "\nFound");
   }

      /* ----------------------------------------------------- */
      /* Second search                                         */
      /* ----------------------------------------------------- */

   d.size = 1;
   sprintf (buff, "1000");
   d.s = buff;

   fprintf (stdout, "\nSearching (from the beginning) for element containing [%s]", d.s);
   element = list_find (&liste, NULL, &d, &prec);
   if (element == NULL)
   {
     fprintf (stdout, "\nNot found");
   }
   else
   {
     fprintf (stdout, "\nFound");
   }

      /* ----------------------------------------------------- */
      /* Third search                                          */
      /* ----------------------------------------------------- */

   d.size = 1;
   sprintf (buff, "4");
   d.s = buff;

   element = list_find (&liste, NULL, &d, &prec);
   if (element != NULL)
   {
      d.size = 1;
      sprintf (buff, "1000");
      d.s = buff;

      fprintf (stdout, "\nSearching (from the element that contains \"4\") for element containing [%s]", d.s);
      element = list_find (&liste, NULL, &d, &prec);
      if (element == NULL)
      {
        fprintf (stdout, "\nNot found");
      }
      else
      {
        fprintf (stdout, "\nFound");
      }
   }

   /* -------------------------------------------------------- */
   /* Remove element after "4"                                 */
   /* -------------------------------------------------------- */

   d.size = 1;
   sprintf (buff, "4");
   d.s = buff;

   element = list_find (&liste, NULL, &d, &prec);
   if (element != NULL)
   {
     fprintf (stdout, "\nRemoving element next to the element that contains 4");
     if (list_rem_next (&liste, element, (void**)&data) == 0)
     {
       fprintf (stdout, "\nElement removed [%s]", data->s);

       /* ---------------------------------------------------- */
       /* Don't forget to free the data's memory !!!           */
       /* ---------------------------------------------------- */

       free_data (data);
     }
     else
     {
       fprintf (stdout, "\nNo element after the one that contains \"4\"");
     }
   }

   /* -------------------------------------------------------- */
   /* Now remove everything from the start of the list         */
   /* -------------------------------------------------------- */

   fprintf (stdout, "\nRemove everything from the start of the list\n");

   while (list_size(&liste) > 0)
   {
     if (list_rem_next (&liste, list_head(&liste), (void**)&data) == 0)
     {
       fprintf (stdout, "\nElement removed [%s], left %d", data->s, list_size(&liste));
       free_data (data);
     }
     else
     {
       fprintf (stdout, "\nNo element removed");
       break;
     }
   }

   if (list_rem_next (&liste, NULL, (void**)&data) == 0)
   {
     fprintf (stdout, "\nElement removed [%s], left %d", data->s, list_size(&liste));
     free_data (data);
   }
   else
   {
     fprintf (stdout, "\nNo element removed");
   }

   /* -------------------------------------------------------- */
   /* Fill linked list with elements                           */
   /* -------------------------------------------------------- */

   fprintf (stdout, "\nFill the linked list with data\n");

   for (i=0; i<max; i++)
   {
     /* ------------------------------------------------------ */
     /* Allocate new data structure                            */
     /* ------------------------------------------------------ */

     data = (struct sdata*)malloc(sizeof(struct sdata));
     if (data == NULL)
     {
        fprintf (stderr, "\nError: can not allocate memory\n");
        return 1;
     }

     /* ------------------------------------------------------ */
     /* initialize the newly allocated data structure          */
     /* ------------------------------------------------------ */

     sprintf (buff, "%d", i);
     data->size = strlen(buff);

     data->s = (char*)malloc((data->size+1)*sizeof(char));
     if (data->s == NULL)
     {
        fprintf (stderr, "\nError: can not allocate memory\n");
        return 1;
     }

     strcpy (data->s, buff);

     /* ------------------------------------------------------ */
     /* Add a new element to the linked list                   */
     /* ------------------------------------------------------ */

     if (list_ins_next (&liste, list_tail(&liste), data) == -1)
     {
        fprintf (stderr, "\nError: can not allocate memory\n");
        return 1;
     }
   }

   /* -------------------------------------------------------- */
   /* Now remove everything from the element just before the   */
   /* end of the list                                          */
   /* -------------------------------------------------------- */

   fprintf (stdout, "\nRemove everything from the element just before the end of the list\n");

   while (list_size(&liste) > 0)
   {
     if (list_rem_next (&liste, list_prev(list_tail(&liste)), (void**)&data) == 0)
     {
       fprintf (stdout, "\nElement removed [%s], left %d", data->s, list_size(&liste));
       free_data (data);
     }
     else
     {
       fprintf (stdout, "\nNo element removed");
       break;
     }
   }

   /* -------------------------------------------------------- */
   /* Fill linked list with 100 elements                       */
   /* -------------------------------------------------------- */

   fprintf (stdout, "\nFill the linked list with data\n");

   for (i=0; i<100; i++)
   {
     /* ------------------------------------------------------ */
     /* Allocate new data structure                            */
     /* ------------------------------------------------------ */

     data = (struct sdata*)malloc(sizeof(struct sdata));
     if (data == NULL)
     {
        fprintf (stderr, "\nError: can not allocate memory\n");
        return 1;
     }

     /* ------------------------------------------------------ */
     /* initialize the newly allocated data structure          */
     /* ------------------------------------------------------ */

     sprintf (buff, "%d", i);
     data->size = strlen(buff);

     data->s = (char*)malloc((data->size+1)*sizeof(char));
     if (data->s == NULL)
     {
        fprintf (stderr, "\nError: can not allocate memory\n");
        return 1;
     }

     strcpy (data->s, buff);

     /* ------------------------------------------------------ */
     /* Add a new element to the linked list                   */
     /* ------------------------------------------------------ */

     if (list_ins_next (&liste, list_tail(&liste), data) == -1)
     {
        fprintf (stderr, "\nError: can not allocate memory\n");
        return 1;
     }
   }

   /* -------------------------------------------------------- */
   /* Remove 20 elements from element "20"                     */
   /* -------------------------------------------------------- */

   fprintf (stdout, "\nDelete 21, 22, 23, 24, ... \n");

   for (i=21; i<41; i++)
   {
     /* ------------------------------------------------------ */
     /* Find the element                                       */
     /* ------------------------------------------------------ */

     d.size = 1;
     sprintf (buff, "%d", i);
     d.s = buff;

     fprintf (stdout, "\nSearching (from the beginning) for element containing [%s]", d.s);
     element = list_find (&liste, NULL, &d, &prec);
     if (element == NULL)
     {
       fprintf (stdout, "\nNot found");
     }
     else
     {
       fprintf (stdout, "\nFound");
     }

     /* ------------------------------------------------------ */
     /* Delete element just before the found element           */
     /* ------------------------------------------------------ */

     if (list_rem_next (&liste, list_prev(element), (void**)&data) == 0)
     {
       fprintf (stdout, "\nElement removed [%s], left %d", data->s, list_size(&liste));
       free_data (data);
     }
     else
     {
       fprintf (stdout, "\nNo element removed");
       break;
     }
   }

   /* -------------------------------------------------------- */
   /* Print the linked list from the start to the end          */
   /* -------------------------------------------------------- */

   fprintf (stdout, "\nPrint the content of the linked list from the start to the end\n");

   element = list_head (&liste);

   while (element != NULL)
   {
     fprintf (stdout, "\n%s", ((struct sdata*)list_data(element))->s);
     element = list_next(element);
   }

   /* -------------------------------------------------------- */
   /* Print the linked list from the end to the start          */
   /* -------------------------------------------------------- */

   fprintf (stdout, "\nPrint the content of the linked list from the end to the start\n");

   element = list_tail (&liste);

   while (element != NULL)
   {
     fprintf (stdout, "\n%s", ((struct sdata*)list_data(element))->s);
     element = list_prev(element);
   }

   /* -------------------------------------------------------- */
   /* Free all allocated memory and return                     */
   /* -------------------------------------------------------- */

   list_destroy(&liste);

   return 0;
}

