#include <stdio.h>
#include <unistd.h>
#include <errno.h>
#include <stdlib.h>
#include "flock.h"


#define BUFF_SIZE 10000

int main (int argc, char *argv[])
{
  int  flock_fd;
  int  i, num, cr;
  char buff[BUFF_SIZE];

  if (argc != 4)
  {
    fprintf (stderr, "\nUsage: log_rotate.test <log file> <file to rename> <wr time>\n\n");
    return 1;
  }

  num = atoi(argv[3]);
  memset ((void*)buff, (int)'A', BUFF_SIZE);

  /**************************************/
  /* Open log file                      */
  /**************************************/

  fprintf (stdout, "\nOpening file [%s]", argv[1]); fflush(stdout);

  flock_fd = open (argv[1], O_WRONLY | O_CREAT | O_APPEND, S_IRWXU);
  if (flock_fd == -1)
  {
    fprintf (stderr, "\nCan not open file [%s]", argv[1]);
    return 1;
  }

  /**************************************/
  /* Lock log file                      */
  /**************************************/

  fprintf (stdout, "\nLocking file [%s]", argv[1]); fflush(stdout);

  if (lock_file (flock_fd) == 1)
  {
    fprintf (stderr, "\nError while locking [%s]", argv[1]);
    return 1;
  }

  /**************************************/
  /* Write data                         */
  /**************************************/

  fprintf (stdout, "\nWrite data in [%s] for [%d seconds]", argv[1], num); fflush(stdout);

  for (i=0; i<num; i++)
  {
    cr = write (flock_fd, buff, BUFF_SIZE);
    if (cr != BUFF_SIZE)
    {
      fprintf (stderr, "\nError while writing data into [%s]", argv[1]);
      return 1;
    }
    sleep (1);
  }
 
  /**************************************/
  /* Unlocking log file                 */
  /**************************************/

  fprintf (stdout, "\nUnlocking file [%s]", argv[1]); fflush(stdout);

  if (unlock_file (flock_fd) == 1)
  {
    fprintf (stderr, "\nError while unlocking [%s]", argv[1]);
    return 1;
  }

  /**************************************/
  /* Close log file                     */
  /**************************************/

  fprintf (stdout, "\nClose file [%s]", argv[1]); fflush(stdout);

  if (close (flock_fd) == -1)
  {
    fprintf (stderr, "\nError while closing [%s] -- [%s]", argv[1], strerror(errno));
    return 1;
  }

  /**************************************/
  /* Try to rename file into "toto"     */
  /**************************************/

  fprintf (stdout, "\nRenaming [%s] into toto", argv[2]); fflush(stdout);

  if (rename (argv[2], "toto") == -1)
  {
    fprintf (stderr, "\nError while renaming [%s] into toto -- [%s]", argv[2], strerror(errno));
    return 1;
  }


  return 0;
}
