

#include <signal.h>
#include <stdio.h>
#include <string.h>
#include "my_smtp.h"


#define CONNECT_TIMEOUT_SEC    3
#define CONNECT_TIMEOUT_MIC    10
#define READ_TIMEOUT_SEC       3
#define READ_TIMEOUT_MIC       10
#define CONNECT_ID             "HELO fr.tiscali.com\r\n"
#define SERVER_HOST_NAME       "smtp.tiscali.fr"
#define SERVER_PORT            25
#define FROM_TAG               "test@tiscali.fr"
#define RCPT_TAG               "Denis.BEURIVE@fr.tiscali.com"
#define RCPT_NUMBER            1
#define MAIL_DATA              "test\r\n\r\n.\r\n"

#define MAX_BUFFER_SIZE        1024



static void dummy_handler(int signo) {}


int main (int argc, char *argv[])
{
  struct sigaction              new_sig_settings;
  int                           server_socket, cr;
  struct smtp_connexion_config  connect_config;
  struct smtp_res               res;
  struct smtp_mail              my_mail;
  char                          iobuff[MAX_BUFFER_SIZE];
  

  /* ---------------------------------------------- */
  /* Set dummy signal handler for SIGALRM           */
  /* ---------------------------------------------- */

  new_sig_settings.sa_handler = dummy_handler;

  if (sigemptyset (&(new_sig_settings.sa_mask)) == -1)
  {
    fprintf (stderr, "\n\nERROR: Can not set signal handler for SIGALRM\n\n");
    return 1;
  }

  if (sigaddset (&(new_sig_settings.sa_mask), SIGALRM) == -1)
  {
    fprintf (stderr, "\n\nERROR: Can not set signal handler for SIGALRM\n\n");
    return 1;
  }

  new_sig_settings.sa_flags = 0;

  if (sigaction(SIGALRM, &new_sig_settings, 0) == -1)
  {
    fprintf (stderr, "\n\nERROR: Can not set signal handler for SIGALRM\n\n");
    return 1;
  }

  /* ---------------------------------------------- */
  /* Connect to SMTP server                         */
  /* ---------------------------------------------- */

  connect_config.port                  = SERVER_PORT;
  connect_config.connect_timeout_sec   = CONNECT_TIMEOUT_SEC;
  connect_config.connect_timeout_micro = CONNECT_TIMEOUT_MIC;
  connect_config.read_timeout_sec      = READ_TIMEOUT_SEC;
  connect_config.read_timeout_micro    = READ_TIMEOUT_MIC;
  strcpy (connect_config.host, SERVER_HOST_NAME);
  strcpy (connect_config.id, CONNECT_ID);
 
  server_socket = client_smtp_connect (
                                        &connect_config,
                                        &res
                                      );
  if (server_socket == 1)
  {
    fprintf (stderr, "\n\nERROR: Can not connect to the SMTP server");
    fprintf (stderr, "\nerror code is %d\n\n", res.error_code);
    return 1;
  }

  /* ---------------------------------------------- */
  /* Send an email                                  */
  /* ---------------------------------------------- */
 
  strcpy (my_mail.from,        FROM_TAG);
  strcpy ((my_mail.rcpt)[0],   RCPT_TAG);
  strcpy (iobuff,              MAIL_DATA);

  my_mail.data        = iobuff;
  my_mail.rcpt_number = RCPT_NUMBER;

  cr = smtp_send_email (
                         &connect_config,
                         &my_mail,
                         &res
                       );

  if (cr == -1)
  {
    fprintf (stderr, "\n\nERROR: Can not send mail to the SMTP server");
    fprintf (stderr, "\nerror code is %d\n\n", res.error_code);
    return 1;
  }

  /* ---------------------------------------------- */
  /* Close SMTP connexion                           */ 
  /* ---------------------------------------------- */

  smtp_disconnect();

  return 0;
}
